@@ -17,7 +17,11 @@ from axes.handlers.proxy import AxesProxyHandler
 log = getLogger(settings.AXES_LOGGER)
 
 
-user_locked_out = Signal(providing_args=["request", "username", "ip_address"])
+# This signal provides the following arguments to any listeners:
+# request - The current Request object.
+# username - The username of the User who has been locked out.
+# ip_address - The IP of the user who has been locked out.
+user_locked_out = Signal()
 
 
 @receiver(user_login_failed)
