@@ -13,6 +13,7 @@ from django.contrib.auth import get_user_model, login, logout
 
 from axes.conf import settings
 from axes.models import AccessAttempt
+from axes.helpers import get_cache, make_cache_key_list
 from axes.tests.base import AxesTestCase
 
 
@@ -54,7 +55,7 @@ class DjangoTestClientLoginTestCase(DjangoLoginTestCase):
         self.client.force_login(self.user)
 
 
-class LoginTestCase(AxesTestCase):
+class DatabaseLoginTestCase(AxesTestCase):
     """
     Test for lockouts under different configurations and circumstances to prevent false positives and false negatives.
 
@@ -64,8 +65,10 @@ class LoginTestCase(AxesTestCase):
 
     IP_1 = "10.1.1.1"
     IP_2 = "10.2.2.2"
+    IP_3 = "10.2.2.3"
     USER_1 = "valid-user-1"
     USER_2 = "valid-user-2"
+    USER_3 = "valid-user-3"
     EMAIL_1 = "valid-email-1@example.com"
     EMAIL_2 = "valid-email-2@example.com"
 
@@ -78,6 +81,7 @@ class LoginTestCase(AxesTestCase):
     WRONG_PASSWORD = "wrong-password"
     LOCKED_MESSAGE = "Account locked: too many login attempts."
     LOGIN_FORM_KEY = '<input type="submit" value="Log in" />'
+    ATTEMPT_NOT_BLOCKED = 200
     ALLOWED = 302
     BLOCKED = 403
 
@@ -153,6 +157,9 @@ class LoginTestCase(AxesTestCase):
             response = self.login()
             self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
 
+    def attempt_count(self):
+        return AccessAttempt.objects.count()
+
     @override_settings(AXES_RESET_ON_SUCCESS=False)
     def test_reset_on_success_false(self):
         self.almost_lockout()
@@ -160,19 +167,19 @@ class LoginTestCase(AxesTestCase):
 
         response = self.login()
         self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
-        self.assertTrue(AccessAttempt.objects.count())
+        self.assertTrue(self.attempt_count())
 
     @override_settings(AXES_RESET_ON_SUCCESS=True)
     def test_reset_on_success_true(self):
         self.almost_lockout()
-        self.assertTrue(AccessAttempt.objects.count())
+        self.assertTrue(self.attempt_count())
 
         self.login(is_valid_username=True, is_valid_password=True)
-        self.assertFalse(AccessAttempt.objects.count())
+        self.assertFalse(self.attempt_count())
 
         response = self.lockout()
         self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
-        self.assertTrue(AccessAttempt.objects.count())
+        self.assertTrue(self.attempt_count())
 
     @override_settings(AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP=True)
     def test_lockout_by_combination_user_and_ip(self):
@@ -479,6 +486,36 @@ class LoginTestCase(AxesTestCase):
         response = self._login(self.USER_2, self.VALID_PASSWORD, ip_addr=self.IP_2)
         self.assertEqual(response.status_code, self.ALLOWED)
 
+    @override_settings(
+        AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP=True, AXES_FAILURE_LIMIT=2
+    )
+    def test_lockout_by_user_and_ip_allows_when_diff_user_same_ip_using_cache_multiple_attempts(
+        self
+    ):
+        # User 1 is locked out from IP 1.
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, self.IP_1)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # Second attempt from different IP
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, self.IP_2)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # Second attempt from same IP, different username
+        response = self._login(self.USER_2, self.WRONG_PASSWORD, self.IP_1)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # User 1 is blocked from IP 1
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, ip_addr=self.IP_1)
+        self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
+
+        # User 1 is blocked from IP 2
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, ip_addr=self.IP_2)
+        self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
+
+        # User 2 can still login from IP 2, only he has 1 attempt left
+        response = self._login(self.USER_2, self.VALID_PASSWORD, ip_addr=self.IP_2)
+        self.assertEqual(response.status_code, self.ALLOWED)
+
     @override_settings(AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP=True)
     def test_lockout_by_user_and_ip_with_empty_username_allows_other_users_using_cache(
         self,
@@ -489,3 +526,119 @@ class LoginTestCase(AxesTestCase):
         # Still possible to access the login page
         response = self.client.get(reverse("admin:login"), REMOTE_ADDR=self.IP_1)
         self.assertContains(response, self.LOGIN_FORM_KEY, status_code=200, html=True)
+
+    # Test for true and false positives when blocking by user or IP together.
+    # With cache enabled. When AXES_LOCK_OUT_BY_USER_OR_IP = True
+    @override_settings(AXES_LOCK_OUT_BY_USER_OR_IP=True)
+    def test_lockout_by_user_or_ip_blocks_when_same_user_same_ip_using_cache(self):
+        # User 1 is locked out from IP 1.
+        self._lockout_user1_from_ip1()
+
+        # User 1 is still blocked from IP 1.
+        response = self._login(self.USER_1, self.VALID_PASSWORD, ip_addr=self.IP_1)
+        self.assertEqual(response.status_code, self.BLOCKED)
+
+    @override_settings(AXES_LOCK_OUT_BY_USER_OR_IP=True)
+    def test_lockout_by_user_or_ip_allows_when_same_user_diff_ip_using_cache(self):
+        # User 1 is locked out from IP 1.
+        self._lockout_user1_from_ip1()
+
+        # User 1 is blocked out from IP 1
+        response = self._login(self.USER_1, self.VALID_PASSWORD, ip_addr=self.IP_2)
+        self.assertEqual(response.status_code, self.BLOCKED)
+
+    @override_settings(AXES_LOCK_OUT_BY_USER_OR_IP=True)
+    def test_lockout_by_user_or_ip_allows_when_diff_user_same_ip_using_cache(self):
+        # User 1 is locked out from IP 1.
+        self._lockout_user1_from_ip1()
+
+        # User 2 can still login from IP 1.
+        response = self._login(self.USER_2, self.VALID_PASSWORD, ip_addr=self.IP_1)
+        self.assertEqual(response.status_code, self.BLOCKED)
+
+    @override_settings(AXES_LOCK_OUT_BY_USER_OR_IP=True, AXES_FAILURE_LIMIT=3)
+    def test_lockout_by_user_or_ip_allows_when_diff_user_same_ip_using_cache_multiple_attempts(
+        self
+    ):
+        # User 1 is locked out from IP 1.
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, self.IP_1)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # Second attempt from different IP
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, self.IP_2)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # User 1 is blocked on all IPs, he reached 2 attempts
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, ip_addr=self.IP_2)
+        self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, ip_addr=self.IP_3)
+        self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
+
+        # IP 1 has still one attempt left
+        response = self._login(self.USER_2, self.WRONG_PASSWORD, self.IP_1)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # But now IP 1 is blocked for all attempts
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, ip_addr=self.IP_1)
+        self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
+        response = self._login(self.USER_2, self.WRONG_PASSWORD, ip_addr=self.IP_1)
+        self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
+        response = self._login(self.USER_3, self.WRONG_PASSWORD, ip_addr=self.IP_1)
+        self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
+
+    @override_settings(AXES_LOCK_OUT_BY_USER_OR_IP=True, AXES_FAILURE_LIMIT=3)
+    def test_lockout_by_user_or_ip_allows_when_diff_user_same_ip_using_cache_multiple_failed_attempts(
+        self
+    ):
+        """ Test, if the failed attempts make also impact on the attempt count """
+        # User 1 is locked out from IP 1.
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, self.IP_1)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # Second attempt from different IP
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, self.IP_2)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # Second attempt from same IP, different username
+        response = self._login(self.USER_2, self.WRONG_PASSWORD, self.IP_1)
+        self.assertEqual(response.status_code, self.ATTEMPT_NOT_BLOCKED)
+
+        # User 1 is blocked from IP 2
+        response = self._login(self.USER_1, self.WRONG_PASSWORD, ip_addr=self.IP_2)
+        self.assertContains(response, self.LOCKED_MESSAGE, status_code=self.BLOCKED)
+
+        # On IP 2 it is only 2. attempt, for user 2 it is also 2. attempt -> allow log in
+        response = self._login(self.USER_2, self.VALID_PASSWORD, ip_addr=self.IP_2)
+        self.assertEqual(response.status_code, self.ALLOWED)
+
+    @override_settings(AXES_LOCK_OUT_BY_USER_OR_IP=True)
+    def test_lockout_by_user_or_ip_allows_when_diff_user_diff_ip_using_cache(self):
+        # User 1 is locked out from IP 1.
+        self._lockout_user1_from_ip1()
+
+        # User 2 can still login from IP 2.
+        response = self._login(self.USER_2, self.VALID_PASSWORD, ip_addr=self.IP_2)
+        self.assertEqual(response.status_code, self.ALLOWED)
+
+    @override_settings(AXES_LOCK_OUT_BY_USER_OR_IP=True)
+    def test_lockout_by_user_or_ip_with_empty_username_allows_other_users_using_cache(
+        self,
+    ):
+        # User with empty username is locked out from IP 1.
+        self._lockout_user_from_ip(username="", ip_addr=self.IP_1)
+
+        # Still possible to access the login page
+        response = self.client.get(reverse("admin:login"), REMOTE_ADDR=self.IP_1)
+        self.assertContains(response, self.LOGIN_FORM_KEY, status_code=200, html=True)
+
+
+# Test the same logic with cache handler
+@override_settings(AXES_HANDLER="axes.handlers.cache.AxesCacheHandler")
+class CacheLoginTestCase(DatabaseLoginTestCase):
+    def attempt_count(self):
+        cache = get_cache()
+        keys = cache._cache
+        return len(keys)
+
+    def reset(self, **kwargs):
+        get_cache().delete(make_cache_key_list([kwargs])[0])
